"""Contains all the data models used in inputs/outputs"""

from .attachment import Attachment
from .attachment_create_from_url import AttachmentCreateFromUrl
from .comment import Comment
from .comment_create import CommentCreate
from .concise_doc import ConciseDoc
from .concise_task import ConciseTask
from .concise_task_custom_properties_type_0 import ConciseTaskCustomPropertiesType0
from .dartboard import Dartboard
from .doc import Doc
from .doc_create import DocCreate
from .doc_update import DocUpdate
from .folder import Folder
from .list_comments_o_item import ListCommentsOItem
from .list_docs_o_item import ListDocsOItem
from .list_tasks_o_item import ListTasksOItem
from .paginated_comment_list import PaginatedCommentList
from .paginated_comment_list_meta_type_0 import PaginatedCommentListMetaType0
from .paginated_comment_list_meta_type_0_applied_default_filters import (
    PaginatedCommentListMetaType0AppliedDefaultFilters,
)
from .paginated_concise_doc_list import PaginatedConciseDocList
from .paginated_concise_doc_list_meta_type_0 import PaginatedConciseDocListMetaType0
from .paginated_concise_doc_list_meta_type_0_applied_default_filters import (
    PaginatedConciseDocListMetaType0AppliedDefaultFilters,
)
from .paginated_concise_task_list import PaginatedConciseTaskList
from .paginated_concise_task_list_meta_type_0 import PaginatedConciseTaskListMetaType0
from .paginated_concise_task_list_meta_type_0_applied_default_filters import (
    PaginatedConciseTaskListMetaType0AppliedDefaultFilters,
)
from .priority import Priority
from .skill import Skill
from .task import Task
from .task_create import TaskCreate
from .task_create_custom_properties_type_0 import TaskCreateCustomPropertiesType0
from .task_custom_properties_type_0 import TaskCustomPropertiesType0
from .task_move import TaskMove
from .task_relationships_type_0 import TaskRelationshipsType0
from .task_time_tracking_create import TaskTimeTrackingCreate
from .task_update import TaskUpdate
from .task_update_custom_properties_type_0 import TaskUpdateCustomPropertiesType0
from .time_tracking_entry import TimeTrackingEntry
from .user import User
from .user_space_configuration import UserSpaceConfiguration
from .user_space_configuration_custom_property_checkbox_type_def import (
    UserSpaceConfigurationCustomPropertyCheckboxTypeDef,
)
from .user_space_configuration_custom_property_dates_type_def import (
    UserSpaceConfigurationCustomPropertyDatesTypeDef,
)
from .user_space_configuration_custom_property_multiselect_type_def import (
    UserSpaceConfigurationCustomPropertyMultiselectTypeDef,
)
from .user_space_configuration_custom_property_number_type_def import (
    UserSpaceConfigurationCustomPropertyNumberTypeDef,
)
from .user_space_configuration_custom_property_number_type_def_custom_property_number_format_type_def import (
    UserSpaceConfigurationCustomPropertyNumberTypeDefCustomPropertyNumberFormatTypeDef,
)
from .user_space_configuration_custom_property_select_type_def import (
    UserSpaceConfigurationCustomPropertySelectTypeDef,
)
from .user_space_configuration_custom_property_status_type_def import (
    UserSpaceConfigurationCustomPropertyStatusTypeDef,
)
from .user_space_configuration_custom_property_text_type_def import (
    UserSpaceConfigurationCustomPropertyTextTypeDef,
)
from .user_space_configuration_custom_property_time_tracking_type_def import (
    UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef,
)
from .user_space_configuration_custom_property_user_type_def import (
    UserSpaceConfigurationCustomPropertyUserTypeDef,
)
from .view import View
from .wrapped_comment import WrappedComment
from .wrapped_comment_create import WrappedCommentCreate
from .wrapped_dartboard import WrappedDartboard
from .wrapped_doc import WrappedDoc
from .wrapped_doc_create import WrappedDocCreate
from .wrapped_doc_update import WrappedDocUpdate
from .wrapped_folder import WrappedFolder
from .wrapped_help_center_articles import WrappedHelpCenterArticles
from .wrapped_skill import WrappedSkill
from .wrapped_task import WrappedTask
from .wrapped_task_create import WrappedTaskCreate
from .wrapped_task_update import WrappedTaskUpdate
from .wrapped_view import WrappedView

__all__ = (
    "Attachment",
    "AttachmentCreateFromUrl",
    "Comment",
    "CommentCreate",
    "ConciseDoc",
    "ConciseTask",
    "ConciseTaskCustomPropertiesType0",
    "Dartboard",
    "Doc",
    "DocCreate",
    "DocUpdate",
    "Folder",
    "ListCommentsOItem",
    "ListDocsOItem",
    "ListTasksOItem",
    "PaginatedCommentList",
    "PaginatedCommentListMetaType0",
    "PaginatedCommentListMetaType0AppliedDefaultFilters",
    "PaginatedConciseDocList",
    "PaginatedConciseDocListMetaType0",
    "PaginatedConciseDocListMetaType0AppliedDefaultFilters",
    "PaginatedConciseTaskList",
    "PaginatedConciseTaskListMetaType0",
    "PaginatedConciseTaskListMetaType0AppliedDefaultFilters",
    "Priority",
    "Skill",
    "Task",
    "TaskCreate",
    "TaskCreateCustomPropertiesType0",
    "TaskCustomPropertiesType0",
    "TaskMove",
    "TaskRelationshipsType0",
    "TaskTimeTrackingCreate",
    "TaskUpdate",
    "TaskUpdateCustomPropertiesType0",
    "TimeTrackingEntry",
    "User",
    "UserSpaceConfiguration",
    "UserSpaceConfigurationCustomPropertyCheckboxTypeDef",
    "UserSpaceConfigurationCustomPropertyDatesTypeDef",
    "UserSpaceConfigurationCustomPropertyMultiselectTypeDef",
    "UserSpaceConfigurationCustomPropertyNumberTypeDef",
    "UserSpaceConfigurationCustomPropertyNumberTypeDefCustomPropertyNumberFormatTypeDef",
    "UserSpaceConfigurationCustomPropertySelectTypeDef",
    "UserSpaceConfigurationCustomPropertyStatusTypeDef",
    "UserSpaceConfigurationCustomPropertyTextTypeDef",
    "UserSpaceConfigurationCustomPropertyTimeTrackingTypeDef",
    "UserSpaceConfigurationCustomPropertyUserTypeDef",
    "View",
    "WrappedComment",
    "WrappedCommentCreate",
    "WrappedDartboard",
    "WrappedDoc",
    "WrappedDocCreate",
    "WrappedDocUpdate",
    "WrappedFolder",
    "WrappedHelpCenterArticles",
    "WrappedSkill",
    "WrappedTask",
    "WrappedTaskCreate",
    "WrappedTaskUpdate",
    "WrappedView",
)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.concise_task import ConciseTask
    from ..models.paginated_concise_task_list_meta_type_0 import (
        PaginatedConciseTaskListMetaType0,
    )


T = TypeVar("T", bound="PaginatedConciseTaskList")


@_attrs_define
class PaginatedConciseTaskList:
    """
    Attributes:
        count (int):  Example: 123.
        results (list['ConciseTask']):
        next_ (Union[None, Unset, str]):  Example: http://api.example.org/accounts/?offset=400&limit=100.
        previous (Union[None, Unset, str]):  Example: http://api.example.org/accounts/?offset=200&limit=100.
        meta (Union['PaginatedConciseTaskListMetaType0', None, Unset]):
    """

    count: int
    results: list["ConciseTask"]
    next_: Union[None, Unset, str] = UNSET
    previous: Union[None, Unset, str] = UNSET
    meta: Union["PaginatedConciseTaskListMetaType0", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.paginated_concise_task_list_meta_type_0 import (
            PaginatedConciseTaskListMetaType0,
        )

        count = self.count

        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()
            results.append(results_item)

        next_: Union[None, Unset, str]
        if isinstance(self.next_, Unset):
            next_ = UNSET
        else:
            next_ = self.next_

        previous: Union[None, Unset, str]
        if isinstance(self.previous, Unset):
            previous = UNSET
        else:
            previous = self.previous

        meta: Union[None, Unset, dict[str, Any]]
        if isinstance(self.meta, Unset):
            meta = UNSET
        elif isinstance(self.meta, PaginatedConciseTaskListMetaType0):
            meta = self.meta.to_dict()
        else:
            meta = self.meta

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "count": count,
                "results": results,
            }
        )
        if next_ is not UNSET:
            field_dict["next"] = next_
        if previous is not UNSET:
            field_dict["previous"] = previous
        if meta is not UNSET:
            field_dict["meta"] = meta

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.concise_task import ConciseTask
        from ..models.paginated_concise_task_list_meta_type_0 import (
            PaginatedConciseTaskListMetaType0,
        )

        d = dict(src_dict)
        count = d.pop("count")

        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = ConciseTask.from_dict(results_item_data)

            results.append(results_item)

        def _parse_next_(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        next_ = _parse_next_(d.pop("next", UNSET))

        def _parse_previous(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        previous = _parse_previous(d.pop("previous", UNSET))

        def _parse_meta(
            data: object,
        ) -> Union["PaginatedConciseTaskListMetaType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                meta_type_0 = PaginatedConciseTaskListMetaType0.from_dict(data)

                return meta_type_0
            except:  # noqa: E722
                pass
            return cast(Union["PaginatedConciseTaskListMetaType0", None, Unset], data)

        meta = _parse_meta(d.pop("meta", UNSET))

        paginated_concise_task_list = cls(
            count=count,
            results=results,
            next_=next_,
            previous=previous,
            meta=meta,
        )

        paginated_concise_task_list.additional_properties = d
        return paginated_concise_task_list

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""These are legacy components that still rely on private API, moved out of dart.py."""

# Required for type hinting compatibility when using Python 3.9
from __future__ import annotations

from functools import wraps
from typing import Any, Callable, Union

import httpx

from .dart import (
    Dart,
    _auth_failure_exit,
    _handle_request_errors,
    _log,
    _make_id,
    _unknown_failure_exit,
)
from .exception import DartException

_CREATE_TRANSACTION_URL_FRAG = "/transactions/create"
_LIST_DARTBOARDS_URL_FRAG = "/dartboards"
_LIST_FOLDERS_URL_FRAG = "/folders"
_REPLICATE_DARTBOARD_URL_FRAG_FMT = "/dartboards/replicate/{dartboard_id}"
_REPLICATE_SPACE_URL_FRAG_FMT = "/spaces/replicate/{space_id}"


def _get_space_url(host: str, space_id: str) -> str:
    return f"{host}/s/{space_id}"


def _get_dartboard_url(host: str, dartboard_id: str) -> str:
    return f"{host}/d/{dartboard_id}"


def _get_folder_url(host: str, folder_id: str) -> str:
    return f"{host}/f/{folder_id}"


def _handle_api_errors(fn: Callable[..., httpx.Response]) -> Callable[..., httpx.Response]:
    @wraps(fn)
    def wrapper(*args, **kwargs):
        response = fn(*args, **kwargs)
        if response.status_code in {401, 403}:
            _auth_failure_exit()
        return response

    return wrapper


def _parse_transaction_response_and_maybe_exit(response: dict, model_kind: str, model_id: str) -> Any:
    if (
        response is None
        or "results" not in response
        or not response["results"]
        or not response["results"][0]["success"]
    ):
        _unknown_failure_exit()
    models = response["results"][0]["models"][f"{model_kind}s"]
    model = next((e for e in models if e["duid"] == model_id), None)
    if model is None:
        _unknown_failure_exit()
    return model


class DartOld(Dart):
    @_handle_api_errors
    @_handle_request_errors
    def transact(self, operations: list[dict], kind: str) -> httpx.Response:
        transaction = {
            "duid": _make_id(),
            "kind": kind,
            "operations": operations,
        }
        request_body = {
            "client_duid": self.get_client_id(),
            "items": [transaction],
        }
        return self._private_api.get_httpx_client().post(_CREATE_TRANSACTION_URL_FRAG, json=request_body)

    @_handle_api_errors
    @_handle_request_errors
    def get(self, url_frag, *args, **kwargs) -> httpx.Response:
        return self._private_api.get_httpx_client().get(url_frag, *args, **kwargs)

    @_handle_api_errors
    @_handle_request_errors
    def post(self, url_frag, *args, **kwargs) -> httpx.Response:
        return self._private_api.get_httpx_client().post(url_frag, *args, **kwargs)


def replicate_space(
    space_id: str,
    *,
    title: Union[str, None] = None,
    abrev: Union[str, None] = None,
    color_hex: Union[str, None] = None,
    accessible_by_team: Union[bool, None] = None,
    accessor_duids: Union[list[str], None] = None,
) -> str:
    dart = DartOld()
    content = {}
    if title is not None:
        content["title"] = title
    if abrev is not None:
        content["abrev"] = abrev
    if color_hex is not None:
        content["colorHex"] = color_hex
    if accessible_by_team is not None:
        content["accessibleByTeam"] = accessible_by_team
    if accessor_duids is not None:
        content["accessorIds"] = accessor_duids
    response = dart.post(_REPLICATE_SPACE_URL_FRAG_FMT.format(space_id=space_id), json=content)
    space_id = response.json()["duid"]

    _log(f"Replicated space at {_get_space_url(dart.get_base_url(), space_id)}")
    _log("Done.")
    return space_id


def get_dartboards(space_id: str, include_special: bool = False) -> list[dict]:
    dart = DartOld()

    response = dart.get(_LIST_DARTBOARDS_URL_FRAG, params={"space_duid": space_id, "limit": 100})
    response_json = response.json()
    dartboards = response_json["results"] if response_json is not None else []
    while response_json.get("next") is not None:
        response = dart.get(response_json["next"])
        response_json = response.json()
        if response_json is not None and "results" in response_json:
            dartboards.extend(response_json["results"])

    if not include_special:
        dartboards = [e for e in dartboards if e["kind"] == "Custom"]

    _log(f"Got {len(dartboards)} dartboards")
    _log("Done.")
    return dartboards


def replicate_dartboard(dartboard_id: str, *, title: Union[str, None] = None) -> str:
    dart = DartOld()
    content = {}
    if title is not None:
        content["title"] = title
    response = dart.post(_REPLICATE_DARTBOARD_URL_FRAG_FMT.format(dartboard_id=dartboard_id), json=content)
    dartboard_id = response.json()["duid"]

    _log(f"Replicated dartboard at {_get_dartboard_url(dart.get_base_url(), dartboard_id)}")
    _log("Done.")
    return dartboard_id


def update_dartboard(dartboard_id: str, *, title: Union[str, None] = None, color_hex: Union[str, None] = None) -> dict:
    dart = DartOld()
    dartboard_update = {}

    if title is not None:
        dartboard_update["title"] = title
    if color_hex is not None:
        dartboard_update["color_hex"] = color_hex

    if not dartboard_update:
        raise DartException("At least one of 'title' or 'color_hex' must be provided to update a dartboard.")

    dartboard_update["duid"] = dartboard_id
    dartboard_update_op = {
        "model": "dartboard",
        "kind": "update",
        "data": dartboard_update,
    }
    response = dart.transact([dartboard_update_op], "dartboard_update")
    response_json = response.json()
    dartboard = _parse_transaction_response_and_maybe_exit(response_json, "dartboard", dartboard_id)

    _log(f"Updated dartboard {dartboard['title']} at {_get_dartboard_url(dart.get_base_url(), dartboard['duid'])}")
    _log("Done.")
    return dartboard


def get_folders(space_id: str, *, include_special: Union[bool, None] = False) -> list[dict]:
    dart = DartOld()

    response = dart.get(_LIST_FOLDERS_URL_FRAG, params={"space_duid": space_id, "limit": 50})
    response_json = response.json()
    folders = response_json["results"] if response_json is not None else []
    while response_json.get("next") is not None:
        response = dart.get(response_json["next"])
        response_json = response.json()
        if response_json is not None and "results" in response_json:
            folders.extend(response_json["results"])

    if not include_special:
        folders = [e for e in folders if e["kind"] == "Other"]

    _log(f"Got {len(folders)} folders")
    _log("Done.")
    return folders


def update_folder(folder_id: str, *, title: Union[str, None] = None, color_hex: Union[str, None] = None) -> dict:
    dart = DartOld()
    folder_update = {}

    if title is not None:
        folder_update["title"] = title
    if color_hex is not None:
        folder_update["color_hex"] = color_hex

    if not folder_update:
        raise DartException("At least one of 'title' or 'color_hex' must be provided to update a folder.")

    folder_update["duid"] = folder_id
    folder_update_op = {
        "model": "folder",
        "kind": "update",
        "data": folder_update,
    }
    response = dart.transact([folder_update_op], "folder_update")
    response_json = response.json()
    folder = _parse_transaction_response_and_maybe_exit(response_json, "folder", folder_id)

    _log(f"Updated folder {folder['title']} at {_get_folder_url(dart.get_base_url(), folder['duid'])}")
    _log("Done.")
    return folder



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_tasks_o_item import ListTasksOItem
from ...models.paginated_concise_task_list import PaginatedConciseTaskList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    assignee: Union[Unset, str] = UNSET,
    assignee_id: Union[Unset, str] = UNSET,
    created_at: Union[Unset, datetime.datetime] = UNSET,
    created_at_after: Union[Unset, datetime.datetime] = UNSET,
    created_at_before: Union[Unset, datetime.datetime] = UNSET,
    created_by: Union[Unset, str] = UNSET,
    created_by_id: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_id: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.datetime] = UNSET,
    due_at_after: Union[Unset, datetime.datetime] = UNSET,
    due_at_before: Union[Unset, datetime.datetime] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_completed: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    no_defaults: Union[Unset, bool] = False,
    o: Union[Unset, list[ListTasksOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.datetime] = UNSET,
    start_at_after: Union[Unset, datetime.datetime] = UNSET,
    start_at_before: Union[Unset, datetime.datetime] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_id: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    tag_id: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
    type_: Union[Unset, str] = UNSET,
    type_id: Union[Unset, str] = UNSET,
    updated_at: Union[Unset, datetime.datetime] = UNSET,
    updated_at_after: Union[Unset, datetime.datetime] = UNSET,
    updated_at_before: Union[Unset, datetime.datetime] = UNSET,
    updated_by: Union[Unset, str] = UNSET,
    updated_by_id: Union[Unset, str] = UNSET,
    view: Union[Unset, str] = UNSET,
    view_id: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["assignee"] = assignee

    params["assignee_id"] = assignee_id

    json_created_at: Union[Unset, str] = UNSET
    if not isinstance(created_at, Unset):
        json_created_at = created_at.isoformat()
    params["created_at"] = json_created_at

    json_created_at_after: Union[Unset, str] = UNSET
    if not isinstance(created_at_after, Unset):
        json_created_at_after = created_at_after.isoformat()
    params["created_at_after"] = json_created_at_after

    json_created_at_before: Union[Unset, str] = UNSET
    if not isinstance(created_at_before, Unset):
        json_created_at_before = created_at_before.isoformat()
    params["created_at_before"] = json_created_at_before

    params["created_by"] = created_by

    params["created_by_id"] = created_by_id

    params["dartboard"] = dartboard

    params["dartboard_id"] = dartboard_id

    params["description"] = description

    json_due_at: Union[Unset, str] = UNSET
    if not isinstance(due_at, Unset):
        json_due_at = due_at.isoformat()
    params["due_at"] = json_due_at

    json_due_at_after: Union[Unset, str] = UNSET
    if not isinstance(due_at_after, Unset):
        json_due_at_after = due_at_after.isoformat()
    params["due_at_after"] = json_due_at_after

    json_due_at_before: Union[Unset, str] = UNSET
    if not isinstance(due_at_before, Unset):
        json_due_at_before = due_at_before.isoformat()
    params["due_at_before"] = json_due_at_before

    params["ids"] = ids

    params["in_trash"] = in_trash

    params["is_completed"] = is_completed

    params["limit"] = limit

    params["no_defaults"] = no_defaults

    json_o: Union[Unset, list[str]] = UNSET
    if not isinstance(o, Unset):
        json_o = []
        for o_item_data in o:
            o_item = o_item_data.value
            json_o.append(o_item)

    params["o"] = json_o

    params["offset"] = offset

    params["parent_id"] = parent_id

    params["priority"] = priority

    params["size"] = size

    json_start_at: Union[Unset, str] = UNSET
    if not isinstance(start_at, Unset):
        json_start_at = start_at.isoformat()
    params["start_at"] = json_start_at

    json_start_at_after: Union[Unset, str] = UNSET
    if not isinstance(start_at_after, Unset):
        json_start_at_after = start_at_after.isoformat()
    params["start_at_after"] = json_start_at_after

    json_start_at_before: Union[Unset, str] = UNSET
    if not isinstance(start_at_before, Unset):
        json_start_at_before = start_at_before.isoformat()
    params["start_at_before"] = json_start_at_before

    params["status"] = status

    params["status_id"] = status_id

    params["tag"] = tag

    params["tag_id"] = tag_id

    params["title"] = title

    params["type"] = type_

    params["type_id"] = type_id

    json_updated_at: Union[Unset, str] = UNSET
    if not isinstance(updated_at, Unset):
        json_updated_at = updated_at.isoformat()
    params["updated_at"] = json_updated_at

    json_updated_at_after: Union[Unset, str] = UNSET
    if not isinstance(updated_at_after, Unset):
        json_updated_at_after = updated_at_after.isoformat()
    params["updated_at_after"] = json_updated_at_after

    json_updated_at_before: Union[Unset, str] = UNSET
    if not isinstance(updated_at_before, Unset):
        json_updated_at_before = updated_at_before.isoformat()
    params["updated_at_before"] = json_updated_at_before

    params["updated_by"] = updated_by

    params["updated_by_id"] = updated_by_id

    params["view"] = view

    params["view_id"] = view_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/tasks/list",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[PaginatedConciseTaskList]:
    if response.status_code == 200:
        response_200 = PaginatedConciseTaskList.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[PaginatedConciseTaskList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    assignee: Union[Unset, str] = UNSET,
    assignee_id: Union[Unset, str] = UNSET,
    created_at: Union[Unset, datetime.datetime] = UNSET,
    created_at_after: Union[Unset, datetime.datetime] = UNSET,
    created_at_before: Union[Unset, datetime.datetime] = UNSET,
    created_by: Union[Unset, str] = UNSET,
    created_by_id: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_id: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.datetime] = UNSET,
    due_at_after: Union[Unset, datetime.datetime] = UNSET,
    due_at_before: Union[Unset, datetime.datetime] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_completed: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    no_defaults: Union[Unset, bool] = False,
    o: Union[Unset, list[ListTasksOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.datetime] = UNSET,
    start_at_after: Union[Unset, datetime.datetime] = UNSET,
    start_at_before: Union[Unset, datetime.datetime] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_id: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    tag_id: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
    type_: Union[Unset, str] = UNSET,
    type_id: Union[Unset, str] = UNSET,
    updated_at: Union[Unset, datetime.datetime] = UNSET,
    updated_at_after: Union[Unset, datetime.datetime] = UNSET,
    updated_at_before: Union[Unset, datetime.datetime] = UNSET,
    updated_by: Union[Unset, str] = UNSET,
    updated_by_id: Union[Unset, str] = UNSET,
    view: Union[Unset, str] = UNSET,
    view_id: Union[Unset, str] = UNSET,
) -> Response[PaginatedConciseTaskList]:
    """List tasks with powerful filtering options. Filter by dartboard, status, assignee, tags, priority,
    dates, completion state, view, and more. Supports pagination with limit/offset.

    Args:
        assignee (Union[Unset, str]):
        assignee_id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        created_at_after (Union[Unset, datetime.datetime]):
        created_at_before (Union[Unset, datetime.datetime]):
        created_by (Union[Unset, str]):
        created_by_id (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_id (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.datetime]):
        due_at_after (Union[Unset, datetime.datetime]):
        due_at_before (Union[Unset, datetime.datetime]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_completed (Union[Unset, bool]):
        limit (Union[Unset, int]):
        no_defaults (Union[Unset, bool]):  Default: False.
        o (Union[Unset, list[ListTasksOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.datetime]):
        start_at_after (Union[Unset, datetime.datetime]):
        start_at_before (Union[Unset, datetime.datetime]):
        status (Union[Unset, str]):
        status_id (Union[Unset, str]):
        tag (Union[Unset, str]):
        tag_id (Union[Unset, str]):
        title (Union[Unset, str]):
        type_ (Union[Unset, str]):
        type_id (Union[Unset, str]):
        updated_at (Union[Unset, datetime.datetime]):
        updated_at_after (Union[Unset, datetime.datetime]):
        updated_at_before (Union[Unset, datetime.datetime]):
        updated_by (Union[Unset, str]):
        updated_by_id (Union[Unset, str]):
        view (Union[Unset, str]):
        view_id (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedConciseTaskList]
    """

    kwargs = _get_kwargs(
        assignee=assignee,
        assignee_id=assignee_id,
        created_at=created_at,
        created_at_after=created_at_after,
        created_at_before=created_at_before,
        created_by=created_by,
        created_by_id=created_by_id,
        dartboard=dartboard,
        dartboard_id=dartboard_id,
        description=description,
        due_at=due_at,
        due_at_after=due_at_after,
        due_at_before=due_at_before,
        ids=ids,
        in_trash=in_trash,
        is_completed=is_completed,
        limit=limit,
        no_defaults=no_defaults,
        o=o,
        offset=offset,
        parent_id=parent_id,
        priority=priority,
        size=size,
        start_at=start_at,
        start_at_after=start_at_after,
        start_at_before=start_at_before,
        status=status,
        status_id=status_id,
        tag=tag,
        tag_id=tag_id,
        title=title,
        type_=type_,
        type_id=type_id,
        updated_at=updated_at,
        updated_at_after=updated_at_after,
        updated_at_before=updated_at_before,
        updated_by=updated_by,
        updated_by_id=updated_by_id,
        view=view,
        view_id=view_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    assignee: Union[Unset, str] = UNSET,
    assignee_id: Union[Unset, str] = UNSET,
    created_at: Union[Unset, datetime.datetime] = UNSET,
    created_at_after: Union[Unset, datetime.datetime] = UNSET,
    created_at_before: Union[Unset, datetime.datetime] = UNSET,
    created_by: Union[Unset, str] = UNSET,
    created_by_id: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_id: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.datetime] = UNSET,
    due_at_after: Union[Unset, datetime.datetime] = UNSET,
    due_at_before: Union[Unset, datetime.datetime] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_completed: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    no_defaults: Union[Unset, bool] = False,
    o: Union[Unset, list[ListTasksOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.datetime] = UNSET,
    start_at_after: Union[Unset, datetime.datetime] = UNSET,
    start_at_before: Union[Unset, datetime.datetime] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_id: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    tag_id: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
    type_: Union[Unset, str] = UNSET,
    type_id: Union[Unset, str] = UNSET,
    updated_at: Union[Unset, datetime.datetime] = UNSET,
    updated_at_after: Union[Unset, datetime.datetime] = UNSET,
    updated_at_before: Union[Unset, datetime.datetime] = UNSET,
    updated_by: Union[Unset, str] = UNSET,
    updated_by_id: Union[Unset, str] = UNSET,
    view: Union[Unset, str] = UNSET,
    view_id: Union[Unset, str] = UNSET,
) -> Optional[PaginatedConciseTaskList]:
    """List tasks with powerful filtering options. Filter by dartboard, status, assignee, tags, priority,
    dates, completion state, view, and more. Supports pagination with limit/offset.

    Args:
        assignee (Union[Unset, str]):
        assignee_id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        created_at_after (Union[Unset, datetime.datetime]):
        created_at_before (Union[Unset, datetime.datetime]):
        created_by (Union[Unset, str]):
        created_by_id (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_id (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.datetime]):
        due_at_after (Union[Unset, datetime.datetime]):
        due_at_before (Union[Unset, datetime.datetime]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_completed (Union[Unset, bool]):
        limit (Union[Unset, int]):
        no_defaults (Union[Unset, bool]):  Default: False.
        o (Union[Unset, list[ListTasksOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.datetime]):
        start_at_after (Union[Unset, datetime.datetime]):
        start_at_before (Union[Unset, datetime.datetime]):
        status (Union[Unset, str]):
        status_id (Union[Unset, str]):
        tag (Union[Unset, str]):
        tag_id (Union[Unset, str]):
        title (Union[Unset, str]):
        type_ (Union[Unset, str]):
        type_id (Union[Unset, str]):
        updated_at (Union[Unset, datetime.datetime]):
        updated_at_after (Union[Unset, datetime.datetime]):
        updated_at_before (Union[Unset, datetime.datetime]):
        updated_by (Union[Unset, str]):
        updated_by_id (Union[Unset, str]):
        view (Union[Unset, str]):
        view_id (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedConciseTaskList
    """

    return sync_detailed(
        client=client,
        assignee=assignee,
        assignee_id=assignee_id,
        created_at=created_at,
        created_at_after=created_at_after,
        created_at_before=created_at_before,
        created_by=created_by,
        created_by_id=created_by_id,
        dartboard=dartboard,
        dartboard_id=dartboard_id,
        description=description,
        due_at=due_at,
        due_at_after=due_at_after,
        due_at_before=due_at_before,
        ids=ids,
        in_trash=in_trash,
        is_completed=is_completed,
        limit=limit,
        no_defaults=no_defaults,
        o=o,
        offset=offset,
        parent_id=parent_id,
        priority=priority,
        size=size,
        start_at=start_at,
        start_at_after=start_at_after,
        start_at_before=start_at_before,
        status=status,
        status_id=status_id,
        tag=tag,
        tag_id=tag_id,
        title=title,
        type_=type_,
        type_id=type_id,
        updated_at=updated_at,
        updated_at_after=updated_at_after,
        updated_at_before=updated_at_before,
        updated_by=updated_by,
        updated_by_id=updated_by_id,
        view=view,
        view_id=view_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    assignee: Union[Unset, str] = UNSET,
    assignee_id: Union[Unset, str] = UNSET,
    created_at: Union[Unset, datetime.datetime] = UNSET,
    created_at_after: Union[Unset, datetime.datetime] = UNSET,
    created_at_before: Union[Unset, datetime.datetime] = UNSET,
    created_by: Union[Unset, str] = UNSET,
    created_by_id: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_id: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.datetime] = UNSET,
    due_at_after: Union[Unset, datetime.datetime] = UNSET,
    due_at_before: Union[Unset, datetime.datetime] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_completed: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    no_defaults: Union[Unset, bool] = False,
    o: Union[Unset, list[ListTasksOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.datetime] = UNSET,
    start_at_after: Union[Unset, datetime.datetime] = UNSET,
    start_at_before: Union[Unset, datetime.datetime] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_id: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    tag_id: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
    type_: Union[Unset, str] = UNSET,
    type_id: Union[Unset, str] = UNSET,
    updated_at: Union[Unset, datetime.datetime] = UNSET,
    updated_at_after: Union[Unset, datetime.datetime] = UNSET,
    updated_at_before: Union[Unset, datetime.datetime] = UNSET,
    updated_by: Union[Unset, str] = UNSET,
    updated_by_id: Union[Unset, str] = UNSET,
    view: Union[Unset, str] = UNSET,
    view_id: Union[Unset, str] = UNSET,
) -> Response[PaginatedConciseTaskList]:
    """List tasks with powerful filtering options. Filter by dartboard, status, assignee, tags, priority,
    dates, completion state, view, and more. Supports pagination with limit/offset.

    Args:
        assignee (Union[Unset, str]):
        assignee_id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        created_at_after (Union[Unset, datetime.datetime]):
        created_at_before (Union[Unset, datetime.datetime]):
        created_by (Union[Unset, str]):
        created_by_id (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_id (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.datetime]):
        due_at_after (Union[Unset, datetime.datetime]):
        due_at_before (Union[Unset, datetime.datetime]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_completed (Union[Unset, bool]):
        limit (Union[Unset, int]):
        no_defaults (Union[Unset, bool]):  Default: False.
        o (Union[Unset, list[ListTasksOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.datetime]):
        start_at_after (Union[Unset, datetime.datetime]):
        start_at_before (Union[Unset, datetime.datetime]):
        status (Union[Unset, str]):
        status_id (Union[Unset, str]):
        tag (Union[Unset, str]):
        tag_id (Union[Unset, str]):
        title (Union[Unset, str]):
        type_ (Union[Unset, str]):
        type_id (Union[Unset, str]):
        updated_at (Union[Unset, datetime.datetime]):
        updated_at_after (Union[Unset, datetime.datetime]):
        updated_at_before (Union[Unset, datetime.datetime]):
        updated_by (Union[Unset, str]):
        updated_by_id (Union[Unset, str]):
        view (Union[Unset, str]):
        view_id (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedConciseTaskList]
    """

    kwargs = _get_kwargs(
        assignee=assignee,
        assignee_id=assignee_id,
        created_at=created_at,
        created_at_after=created_at_after,
        created_at_before=created_at_before,
        created_by=created_by,
        created_by_id=created_by_id,
        dartboard=dartboard,
        dartboard_id=dartboard_id,
        description=description,
        due_at=due_at,
        due_at_after=due_at_after,
        due_at_before=due_at_before,
        ids=ids,
        in_trash=in_trash,
        is_completed=is_completed,
        limit=limit,
        no_defaults=no_defaults,
        o=o,
        offset=offset,
        parent_id=parent_id,
        priority=priority,
        size=size,
        start_at=start_at,
        start_at_after=start_at_after,
        start_at_before=start_at_before,
        status=status,
        status_id=status_id,
        tag=tag,
        tag_id=tag_id,
        title=title,
        type_=type_,
        type_id=type_id,
        updated_at=updated_at,
        updated_at_after=updated_at_after,
        updated_at_before=updated_at_before,
        updated_by=updated_by,
        updated_by_id=updated_by_id,
        view=view,
        view_id=view_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    assignee: Union[Unset, str] = UNSET,
    assignee_id: Union[Unset, str] = UNSET,
    created_at: Union[Unset, datetime.datetime] = UNSET,
    created_at_after: Union[Unset, datetime.datetime] = UNSET,
    created_at_before: Union[Unset, datetime.datetime] = UNSET,
    created_by: Union[Unset, str] = UNSET,
    created_by_id: Union[Unset, str] = UNSET,
    dartboard: Union[Unset, str] = UNSET,
    dartboard_id: Union[Unset, str] = UNSET,
    description: Union[Unset, str] = UNSET,
    due_at: Union[Unset, datetime.datetime] = UNSET,
    due_at_after: Union[Unset, datetime.datetime] = UNSET,
    due_at_before: Union[Unset, datetime.datetime] = UNSET,
    ids: Union[Unset, str] = UNSET,
    in_trash: Union[Unset, bool] = UNSET,
    is_completed: Union[Unset, bool] = UNSET,
    limit: Union[Unset, int] = UNSET,
    no_defaults: Union[Unset, bool] = False,
    o: Union[Unset, list[ListTasksOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    priority: Union[Unset, str] = UNSET,
    size: Union[Unset, int] = UNSET,
    start_at: Union[Unset, datetime.datetime] = UNSET,
    start_at_after: Union[Unset, datetime.datetime] = UNSET,
    start_at_before: Union[Unset, datetime.datetime] = UNSET,
    status: Union[Unset, str] = UNSET,
    status_id: Union[Unset, str] = UNSET,
    tag: Union[Unset, str] = UNSET,
    tag_id: Union[Unset, str] = UNSET,
    title: Union[Unset, str] = UNSET,
    type_: Union[Unset, str] = UNSET,
    type_id: Union[Unset, str] = UNSET,
    updated_at: Union[Unset, datetime.datetime] = UNSET,
    updated_at_after: Union[Unset, datetime.datetime] = UNSET,
    updated_at_before: Union[Unset, datetime.datetime] = UNSET,
    updated_by: Union[Unset, str] = UNSET,
    updated_by_id: Union[Unset, str] = UNSET,
    view: Union[Unset, str] = UNSET,
    view_id: Union[Unset, str] = UNSET,
) -> Optional[PaginatedConciseTaskList]:
    """List tasks with powerful filtering options. Filter by dartboard, status, assignee, tags, priority,
    dates, completion state, view, and more. Supports pagination with limit/offset.

    Args:
        assignee (Union[Unset, str]):
        assignee_id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        created_at_after (Union[Unset, datetime.datetime]):
        created_at_before (Union[Unset, datetime.datetime]):
        created_by (Union[Unset, str]):
        created_by_id (Union[Unset, str]):
        dartboard (Union[Unset, str]):
        dartboard_id (Union[Unset, str]):
        description (Union[Unset, str]):
        due_at (Union[Unset, datetime.datetime]):
        due_at_after (Union[Unset, datetime.datetime]):
        due_at_before (Union[Unset, datetime.datetime]):
        ids (Union[Unset, str]):
        in_trash (Union[Unset, bool]):
        is_completed (Union[Unset, bool]):
        limit (Union[Unset, int]):
        no_defaults (Union[Unset, bool]):  Default: False.
        o (Union[Unset, list[ListTasksOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        priority (Union[Unset, str]):
        size (Union[Unset, int]):
        start_at (Union[Unset, datetime.datetime]):
        start_at_after (Union[Unset, datetime.datetime]):
        start_at_before (Union[Unset, datetime.datetime]):
        status (Union[Unset, str]):
        status_id (Union[Unset, str]):
        tag (Union[Unset, str]):
        tag_id (Union[Unset, str]):
        title (Union[Unset, str]):
        type_ (Union[Unset, str]):
        type_id (Union[Unset, str]):
        updated_at (Union[Unset, datetime.datetime]):
        updated_at_after (Union[Unset, datetime.datetime]):
        updated_at_before (Union[Unset, datetime.datetime]):
        updated_by (Union[Unset, str]):
        updated_by_id (Union[Unset, str]):
        view (Union[Unset, str]):
        view_id (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedConciseTaskList
    """

    return (
        await asyncio_detailed(
            client=client,
            assignee=assignee,
            assignee_id=assignee_id,
            created_at=created_at,
            created_at_after=created_at_after,
            created_at_before=created_at_before,
            created_by=created_by,
            created_by_id=created_by_id,
            dartboard=dartboard,
            dartboard_id=dartboard_id,
            description=description,
            due_at=due_at,
            due_at_after=due_at_after,
            due_at_before=due_at_before,
            ids=ids,
            in_trash=in_trash,
            is_completed=is_completed,
            limit=limit,
            no_defaults=no_defaults,
            o=o,
            offset=offset,
            parent_id=parent_id,
            priority=priority,
            size=size,
            start_at=start_at,
            start_at_after=start_at_after,
            start_at_before=start_at_before,
            status=status,
            status_id=status_id,
            tag=tag,
            tag_id=tag_id,
            title=title,
            type_=type_,
            type_id=type_id,
            updated_at=updated_at,
            updated_at_after=updated_at_after,
            updated_at_before=updated_at_before,
            updated_by=updated_by,
            updated_by_id=updated_by_id,
            view=view,
            view_id=view_id,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TaskTimeTrackingCreate")


@_attrs_define
class TaskTimeTrackingCreate:
    """
    Attributes:
        user (Union[None, str]): The name or email of the user to attribute the tracked time to or null to use the
            current user.
        started_at (str): The start timestamp for the tracked time entry in ISO 8601 format.
        finished_at (str): The end timestamp for the tracked time entry in ISO 8601 format. Must be after the start
            time.
    """

    user: Union[None, str]
    started_at: str
    finished_at: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        user: Union[None, str]
        user = self.user

        started_at = self.started_at

        finished_at = self.finished_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
                "startedAt": started_at,
                "finishedAt": finished_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_user(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        user = _parse_user(d.pop("user"))

        started_at = d.pop("startedAt")

        finished_at = d.pop("finishedAt")

        task_time_tracking_create = cls(
            user=user,
            started_at=started_at,
            finished_at=finished_at,
        )

        task_time_tracking_create.additional_properties = d
        return task_time_tracking_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="TaskRelationshipsType0")


@_attrs_define
class TaskRelationshipsType0:
    """
    Example:
        {'subtaskIds': ['abcdefghijk1', 'abcdefghijk2'], 'blockerIds': ['abcdefghijk3'], 'blockingIds':
            ['abcdefghijk4'], 'duplicateIds': ['abcdefghijk5'], 'relatedIds': ['abcdefghijk6', 'abcdefghijk7']}

    Attributes:
        subtask_ids (Union[Unset, list[str]]): Array of task IDs that are subtasks of this task. These tasks have a
            parent-child relationship where this task is the parent. Subtasks inherit context from their parent and are
            typically smaller units of work.
        blocker_ids (Union[Unset, list[str]]): Array of task IDs that block this task from being completed. These are
            dependencies that must be resolved/completed before this task can proceed. The blocking tasks have priority over
            this task.
        blocking_ids (Union[Unset, list[str]]): Array of task IDs that are blocked by this task. This task must be
            completed before the blocked tasks can proceed. This task is a dependency for the blocked tasks and has priority
            over them.
        duplicate_ids (Union[Unset, list[str]]): Array of task IDs that are duplicates of this task. These represent the
            same work item and should typically be consolidated to avoid duplicate effort. Only one of the duplicates should
            be completed.
        related_ids (Union[Unset, list[str]]): Array of task IDs that are related to this task. These tasks have some
            contextual relationship but no direct dependency. They may share similar goals, components, or be part of the
            same feature/epic.
    """

    subtask_ids: Union[Unset, list[str]] = UNSET
    blocker_ids: Union[Unset, list[str]] = UNSET
    blocking_ids: Union[Unset, list[str]] = UNSET
    duplicate_ids: Union[Unset, list[str]] = UNSET
    related_ids: Union[Unset, list[str]] = UNSET

    def to_dict(self) -> dict[str, Any]:
        subtask_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.subtask_ids, Unset):
            subtask_ids = self.subtask_ids

        blocker_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.blocker_ids, Unset):
            blocker_ids = self.blocker_ids

        blocking_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.blocking_ids, Unset):
            blocking_ids = self.blocking_ids

        duplicate_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.duplicate_ids, Unset):
            duplicate_ids = self.duplicate_ids

        related_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.related_ids, Unset):
            related_ids = self.related_ids

        field_dict: dict[str, Any] = {}
        field_dict.update({})
        if subtask_ids is not UNSET:
            field_dict["subtaskIds"] = subtask_ids
        if blocker_ids is not UNSET:
            field_dict["blockerIds"] = blocker_ids
        if blocking_ids is not UNSET:
            field_dict["blockingIds"] = blocking_ids
        if duplicate_ids is not UNSET:
            field_dict["duplicateIds"] = duplicate_ids
        if related_ids is not UNSET:
            field_dict["relatedIds"] = related_ids

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        subtask_ids = cast(list[str], d.pop("subtaskIds", UNSET))

        blocker_ids = cast(list[str], d.pop("blockerIds", UNSET))

        blocking_ids = cast(list[str], d.pop("blockingIds", UNSET))

        duplicate_ids = cast(list[str], d.pop("duplicateIds", UNSET))

        related_ids = cast(list[str], d.pop("relatedIds", UNSET))

        task_relationships_type_0 = cls(
            subtask_ids=subtask_ids,
            blocker_ids=blocker_ids,
            blocking_ids=blocking_ids,
            duplicate_ids=duplicate_ids,
            related_ids=related_ids,
        )

        return task_relationships_type_0



from enum import Enum


class UserSpaceConfigurationCustomPropertyNumberTypeDefCustomPropertyNumberFormatTypeDef(str, Enum):
    DOLLARS = "Dollars"
    INTEGER = "Integer"
    PERCENTAGE = "Percentage"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    Any,
    Literal,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="UserSpaceConfigurationCustomPropertyMultiselectTypeDef")


@_attrs_define
class UserSpaceConfigurationCustomPropertyMultiselectTypeDef:
    """
    Attributes:
        name (str):
        type_ (Literal['Multiselect']):
        options (list[str]):
    """

    name: str
    type_: Literal["Multiselect"]
    options: list[str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type_ = self.type_

        options = self.options

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type_,
                "options": options,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        type_ = cast(Literal["Multiselect"], d.pop("type"))
        if type_ != "Multiselect":
            raise ValueError(f"type must match const 'Multiselect', got '{type_}'")

        options = cast(list[str], d.pop("options"))

        user_space_configuration_custom_property_multiselect_type_def = cls(
            name=name,
            type_=type_,
            options=options,
        )

        user_space_configuration_custom_property_multiselect_type_def.additional_properties = d
        return user_space_configuration_custom_property_multiselect_type_def

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#!/usr/bin/env uv run python3
# -*- coding: utf-8 -*-

"""A CLI to interact with the Dart web app."""

# Required for type hinting compatibility when using Python 3.9
from __future__ import annotations

import json
import os
import random
import re
import signal
import string
import subprocess
import sys
from argparse import ArgumentParser
from collections import defaultdict
from datetime import timezone
from functools import wraps
from importlib.metadata import version
from typing import Callable, NoReturn, TypeVar, Union
from webbrowser import open_new_tab

import dateparser
import httpx
import platformdirs
from pick import pick

from .exception import DartException
from .generated import Client, api
from .generated.models import (
    Comment,
    CommentCreate,
    ConciseTask,
    Doc,
    DocCreate,
    DocUpdate,
    PaginatedCommentList,
    PaginatedConciseDocList,
    PaginatedConciseTaskList,
    Priority,
    Task,
    TaskCreate,
    TaskUpdate,
    UserSpaceConfiguration,
    WrappedComment,
    WrappedCommentCreate,
    WrappedDoc,
    WrappedDocCreate,
    WrappedDocUpdate,
    WrappedTask,
    WrappedTaskCreate,
    WrappedTaskUpdate,
)
from .generated.types import UNSET, Response, Unset

_APP = "dart-tools"
_PROG = "dart"

_PROD_HOST = "https://app.dartai.com"
_STAG_HOST = "https://stag.dartai.com"
_DEV_HOST = "http://localhost:5100"
_HOST_MAP = {"prod": _PROD_HOST, "stag": _STAG_HOST, "dev": _DEV_HOST}

# Service commands
_LOGIN_CMD = "login"
_SET_HOST_CMD = "host-set"
_VERSION_CMD = "--version"
# Task commands
_CREATE_TASK_CMD = "task-create"
_UPDATE_TASK_CMD = "task-update"
_DELETE_TASK_CMD = "task-delete"
_BEGIN_TASK_CMD = "task-begin"
# Doc commands
_CREATE_DOC_CMD = "doc-create"
_UPDATE_DOC_CMD = "doc-update"
_DELETE_DOC_CMD = "doc-delete"
# Comment commands
_CREATE_COMMENT_CMD = "comment-create"

_PROFILE_SETTINGS_URL_FRAG = "/?settings=account"
_ROOT_PRIVATE_API_URL_FRAG = "/api/v0"
_ROOT_PUBLIC_API_URL_FRAG = f"{_ROOT_PRIVATE_API_URL_FRAG}/public"
_COPY_BRANCH_URL_FRAG = "/vcs/copy-branch-link"

_AUTH_TOKEN_ENVVAR_KEY = "DART_TOKEN"
_HOST_ENVVAR_KEY = "DART_HOST"
_CONFIG_FPATH = platformdirs.user_config_path(_APP)
_CLIENT_ID_KEY = "clientId"
_HOST_KEY = "host"
_HOSTS_KEY = "hosts"
_AUTH_TOKEN_KEY = "authToken"

_ID_CHARS = string.ascii_lowercase + string.ascii_uppercase + string.digits
_NON_ALPHANUM_RE = re.compile(r"[^a-zA-Z0-9-]+")
_REPEATED_DASH_RE = re.compile(r"-{2,}")
_PRIORITY_MAP: dict[int, str] = {
    0: Priority.CRITICAL,
    1: Priority.HIGH,
    2: Priority.MEDIUM,
    3: Priority.LOW,
}
_SIZES = {1, 2, 3, 5, 8}
_DEFAULT_DARTBOARD = "General/Active"

_VERSION = version(_APP)
_AUTH_TOKEN_ENVVAR = os.environ.get(_AUTH_TOKEN_ENVVAR_KEY)
_HOST_ENVVAR = os.environ.get(_HOST_ENVVAR_KEY)
_DEFAULT_HOST = _HOST_ENVVAR or _PROD_HOST


def _get_help_text(fn: Callable) -> str:
    if fn.__doc__ is None:
        raise ValueError(f"Function {fn.__name__} has no docstring.")
    return fn.__doc__.split("\n")[0].lower()


_HELP_TEXT_TO_COMMAND = {
    _CREATE_TASK_CMD: _get_help_text(api.create_task.sync_detailed),
    _UPDATE_TASK_CMD: _get_help_text(api.update_task.sync_detailed),
    _DELETE_TASK_CMD: _get_help_text(api.delete_task.sync_detailed),
    _CREATE_DOC_CMD: _get_help_text(api.create_doc.sync_detailed),
    _UPDATE_DOC_CMD: _get_help_text(api.update_doc.sync_detailed),
    _DELETE_DOC_CMD: _get_help_text(api.delete_doc.sync_detailed),
    _CREATE_COMMENT_CMD: _get_help_text(api.add_task_comment.sync_detailed),
}

_is_cli = False


# TODO dedupe these functions with other usages elsewhere
def _make_id() -> str:
    return "".join(random.choices(_ID_CHARS, k=12))


def trim_slug_str(s: str, length: int, max_under: Union[int, None] = None) -> str:
    max_under = max_under if max_under is not None else length // 6
    if len(s) <= length:
        return s
    for i in range(1, max_under + 1):
        if s[length - i] == "-":
            return s[: length - i]
    return s[:length]


def slugify_str(s: str, lower: bool = False, trim_kwargs: Union[dict, None] = None) -> str:
    lowered = s.lower() if lower else s
    formatted = _NON_ALPHANUM_RE.sub("-", lowered.replace("'", ""))
    formatted = _REPEATED_DASH_RE.sub("-", formatted).strip("-")
    return trim_slug_str(formatted, **trim_kwargs) if trim_kwargs is not None else formatted


def _run_cmd(cmd: str) -> str:
    return subprocess.check_output(cmd, shell=True).decode()


def _suppress_exception(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except Exception:  # pylint: disable=broad-except
            return None

    return wrapper


def _dart_exit(message: str) -> NoReturn:
    if _is_cli:
        sys.exit(message)
    raise DartException(message)


def _exit_gracefully(_signal_received, _frame) -> None:
    _dart_exit("Quitting.")


def _log(s: str) -> None:
    if not _is_cli:
        return
    print(s)


T = TypeVar("T")


def _get_response_parsed(response: Response[T], not_found_message="Not found.") -> T:
    if response.parsed is not None:
        return response.parsed
    if response.status_code in {401, 403}:
        _auth_failure_exit()
    elif response.status_code == 404:
        _dart_exit(not_found_message)
    try:
        response_content = json.loads(response.content)
        error_message = response_content.get("detail") or " ".join(response_content.get("errors", []))
        _dart_exit(error_message)
    except (json.JSONDecodeError, AttributeError):
        _unknown_failure_exit()


def _handle_request_errors(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except DartException as ex:
            _dart_exit(ex)
        except (httpx.TimeoutException, httpx.RequestError, httpx.ConnectError) as ex:
            _dart_exit(f"Failed to execute API call: {ex}.")

    return wrapper


class _Config:
    def __init__(self):
        self._content = {}
        if os.path.isfile(_CONFIG_FPATH):
            try:
                with open(_CONFIG_FPATH, "r", encoding="UTF-8") as fin:
                    self._content = json.load(fin)
            except OSError:
                pass
        self._content = {
            _CLIENT_ID_KEY: _make_id(),
            _HOST_KEY: _DEFAULT_HOST,
            _HOSTS_KEY: {},
        } | self._content
        self._content[_HOSTS_KEY] = defaultdict(dict, self._content[_HOSTS_KEY])
        self._write()

    def _write(self) -> None:
        try:
            with open(_CONFIG_FPATH, "w+", encoding="UTF-8") as fout:
                json.dump(self._content, fout, indent=2)
        except OSError:
            pass

    @property
    def client_id(self) -> str:
        return self._content[_CLIENT_ID_KEY]

    @property
    def host(self) -> str:
        return self._content[_HOST_KEY]

    @host.setter
    def host(self, v: str) -> None:
        self._content[_HOST_KEY] = v
        self._write()

    def get(self, k: str) -> str | None:
        return self._content[_HOSTS_KEY][self.host].get(k)

    def set(self, k: str, v: str) -> None:
        self._content[_HOSTS_KEY][self.host][k] = v
        self._write()


class Dart:
    def __init__(self, config=None):
        self._config = config or _Config()
        self._init_clients()

    def _init_clients(self) -> None:
        self._private_api = Client(
            base_url=self.get_base_url() + _ROOT_PRIVATE_API_URL_FRAG,
            headers=self.get_headers(),
        )
        self._public_api = Client(
            base_url=self.get_base_url() + _ROOT_PUBLIC_API_URL_FRAG,
            headers=self.get_headers(),
        )

    def get_base_url(self) -> str:
        return self._config.host

    def get_client_id(self) -> str:
        return self._config.client_id

    def get_auth_token(self) -> Union[str, None]:
        result = self._config.get(_AUTH_TOKEN_KEY)
        if result is not None:
            return result
        return _AUTH_TOKEN_ENVVAR

    def get_headers(self) -> dict[str, str]:
        result = {
            "Origin": self._config.host,
            "client-duid": self.get_client_id(),
        }
        if (auth_token := self.get_auth_token()) is not None:
            result["Authorization"] = f"Bearer {auth_token}"
        return result

    def is_logged_in(self) -> bool:
        self._init_clients()
        try:
            config = api.get_config.sync(client=self._public_api)
            if config is None:
                return False
        except:
            return False
        return True

    @_handle_request_errors
    def get_config(self) -> UserSpaceConfiguration:
        response = api.get_config.sync_detailed(client=self._public_api)
        return _get_response_parsed(response)

    @_handle_request_errors
    def create_task(self, body: WrappedTaskCreate) -> WrappedTask:
        response = api.create_task.sync_detailed(client=self._public_api, body=body)
        return _get_response_parsed(response)

    @_handle_request_errors
    def retrieve_task(self, id: str) -> WrappedTask:
        response = api.get_task.sync_detailed(id, client=self._public_api)
        return _get_response_parsed(response, not_found_message=f"Task with ID {id} not found.")

    @_handle_request_errors
    def update_task(self, id: str, body: WrappedTaskUpdate) -> WrappedTask:
        response = api.update_task.sync_detailed(id, client=self._public_api, body=body)
        return _get_response_parsed(response, not_found_message=f"Task with ID {id} not found.")

    @_handle_request_errors
    def delete_task(self, id: str) -> WrappedTask:
        response = api.delete_task.sync_detailed(id, client=self._public_api)
        return _get_response_parsed(response, not_found_message=f"Task with ID {id} not found.")

    @_handle_request_errors
    def list_tasks(self, **kwargs) -> PaginatedConciseTaskList:
        response = api.list_tasks.sync_detailed(client=self._public_api, **kwargs)
        return _get_response_parsed(response)

    @_handle_request_errors
    def create_comment(self, body: WrappedCommentCreate) -> WrappedComment:
        response = api.add_task_comment.sync_detailed(client=self._public_api, body=body)
        return _get_response_parsed(response)

    @_handle_request_errors
    def list_comments(self, **kwargs) -> PaginatedCommentList:
        response = api.list_comments.sync_detailed(client=self._public_api, **kwargs)
        return _get_response_parsed(response)

    @_handle_request_errors
    def create_doc(self, body: WrappedDocCreate) -> WrappedDoc:
        response = api.create_doc.sync_detailed(client=self._public_api, body=body)
        return _get_response_parsed(response)

    @_handle_request_errors
    def retrieve_doc(self, id: str) -> WrappedDoc:
        response = api.get_doc.sync_detailed(id, client=self._public_api)
        return _get_response_parsed(response, not_found_message=f"Doc with ID {id} not found.")

    @_handle_request_errors
    def update_doc(self, id: str, body: WrappedDocUpdate) -> WrappedDoc:
        response = api.update_doc.sync_detailed(id, client=self._public_api, body=body)
        return _get_response_parsed(response, not_found_message=f"Doc with ID {id} not found.")

    @_handle_request_errors
    def delete_doc(self, id: str) -> WrappedDoc:
        response = api.delete_doc.sync_detailed(id, client=self._public_api)
        return _get_response_parsed(response, not_found_message=f"Doc with ID {id} not found.")

    @_handle_request_errors
    def list_docs(self, **kwargs) -> PaginatedConciseDocList:
        response = api.list_docs.sync_detailed(client=self._public_api, **kwargs)
        return _get_response_parsed(response)

    @_handle_request_errors
    def copy_branch_link(self, id: str) -> None:
        self._private_api.get_httpx_client().post(_COPY_BRANCH_URL_FRAG, json={"duid": id})


def make_task_branch_name(email: str, task: ConciseTask | Task) -> str:
    username = slugify_str(email.split("@")[0], lower=True)
    title = slugify_str(task.title, lower=True)
    return trim_slug_str(f"{username}/{task.id}-{title}", length=60)


def get_host() -> str:
    config = _Config()

    host = config.host
    _log(f"Host is {host}")
    _log("Done.")
    return host


def set_host(host: str) -> bool:
    config = _Config()

    new_host = _HOST_MAP.get(host, host)
    config.host = new_host

    _log(f"Set host to {new_host}")
    _log("Done.")
    return True


def _auth_failure_exit() -> NoReturn:
    _dart_exit(
        f"Not logged in, run\n\n  {_PROG} {_LOGIN_CMD}\n\nto log in."
        if _is_cli
        else "Not logged in, either run\n\n  dart.login(token)\n\nor save the token into the DART_TOKEN environment variable."
    )


def _unknown_failure_exit() -> NoReturn:
    _dart_exit("Unknown failure, email\n\n  support@dartai.com\n\nfor help.")


def print_version() -> str:
    result = f"dart-tools version {_VERSION}"
    _log(result)
    return result


@_suppress_exception
def print_version_update_message_maybe() -> None:
    latest = (
        _run_cmd("pip --disable-pip-version-check index versions dart-tools 2>&1")
        .rsplit("LATEST:", maxsplit=1)[-1]
        .split("\n", maxsplit=1)[0]
        .strip()
    )
    if latest == _VERSION or [int(e) for e in latest.split(".")] <= [int(e) for e in _VERSION.split(".")]:
        return

    _log(
        f"A new version of dart-tools is available. Upgrade from {_VERSION} to {latest} with\n\n  pip install --upgrade dart-tools\n"
    )


def is_logged_in(should_raise: bool = False) -> bool:
    dart = Dart()
    result = dart.is_logged_in()

    if not result and should_raise:
        _auth_failure_exit()
    _log(f"You are{'' if result else ' not'} logged in")
    return result


def login(token: str | None = None) -> bool:
    config = _Config()
    dart = Dart(config=config)

    if dart.is_logged_in():
        _log("Already logged in.")
        return True

    _log("Log in to Dart")
    if token is None:
        if not _is_cli:
            _dart_exit("Login failed, token is required.")
        _log("Dart is opening in your browser, log in if needed and copy your authentication token from the page")
        open_new_tab(config.host + _PROFILE_SETTINGS_URL_FRAG)
        token = input("Token: ")

    config.set(_AUTH_TOKEN_KEY, token)

    worked = dart.is_logged_in()
    if not worked:
        _dart_exit("Invalid token.")

    _log("Logged in.")
    return True


def _begin_task(dart: Dart, email: str, task: ConciseTask | Task) -> bool:
    dart = Dart()
    dart.copy_branch_link(task.id)

    branch_name = make_task_branch_name(email, task)

    _log(f"Started work on\n\n  {task.title}\n  {task.html_url}\n  {branch_name}\n")
    return True


def begin_task() -> bool:
    dart = Dart()
    config = dart.get_config()
    user = config.user
    dartboard_maybe = {"dartboard": _DEFAULT_DARTBOARD} if _DEFAULT_DARTBOARD in config.dartboards else {}
    filtered_tasks = dart.list_tasks(assignee=user.email, is_completed=False, **dartboard_maybe).results

    if not filtered_tasks:
        _dart_exit("No active, incomplete tasks found.")

    picked_idx = pick(
        [e.title for e in filtered_tasks],
        "Which of your active, incomplete tasks are you beginning work on?",
        "→",
    )[1]
    assert isinstance(picked_idx, int)

    _begin_task(dart, user.email, filtered_tasks[picked_idx])

    _log("Done.")
    return True


def _normalize_priority(priority_int: Union[int, None, Unset]) -> Union[str, None, Unset]:
    if priority_int in (None, UNSET):
        return priority_int

    if priority_int not in _PRIORITY_MAP:
        _dart_exit(f"Invalid priority {priority_int}. Valid values are {list(_PRIORITY_MAP.keys())}.")

    return _PRIORITY_MAP[priority_int]


def _get_due_at_from_str_arg(due_at_str: Union[str, None, Unset]) -> Union[str, None, Unset]:
    if due_at_str in (None, UNSET):
        return due_at_str

    due_at = dateparser.parse(due_at_str)
    if not due_at:
        _dart_exit(f"Could not parse due date: {due_at_str}.")
    due_at = due_at.replace(hour=0, minute=0, second=0, microsecond=0, tzinfo=timezone.utc).isoformat()

    return due_at


def create_task(
    title: str,
    *,
    dartboard_title: Union[str, Unset] = UNSET,
    status_title: Union[str, Unset] = UNSET,
    assignee_emails: Union[list[str], Unset] = UNSET,
    tag_titles: Union[list[str], Unset] = UNSET,
    priority_int: Union[int, None, Unset] = UNSET,
    size_int: Union[int, None, Unset] = UNSET,
    due_at_str: Union[str, None, Unset] = UNSET,
    should_begin: bool = False,
) -> Task:
    dart = Dart()
    task_create = WrappedTaskCreate(
        item=TaskCreate(
            title,
            dartboard=dartboard_title,
            status=status_title,
            assignees=assignee_emails if assignee_emails is not None else UNSET,
            tags=tag_titles if tag_titles is not None else UNSET,
            priority=_normalize_priority(priority_int),
            size=size_int,
            due_at=_get_due_at_from_str_arg(due_at_str),
        )
    )
    task = dart.create_task(task_create).item
    _log(f"Created task\n\n  {task.title}\n  {task.html_url}\n  ID: {task.id}\n")

    if should_begin:
        user = dart.get_config().user
        _begin_task(dart, user.email, task)

    _log("Done.")
    return task


def update_task(
    id: str,
    *,
    title: Union[Unset, str] = UNSET,
    dartboard_title: Union[str, Unset] = UNSET,
    status_title: Union[str, Unset] = UNSET,
    assignee_emails: Union[list[str], Unset] = UNSET,
    tag_titles: Union[list[str], Unset] = UNSET,
    priority_int: Union[int, None, Unset] = UNSET,
    size_int: Union[int, None, Unset] = UNSET,
    due_at_str: Union[str, None, Unset] = UNSET,
) -> Task:
    dart = Dart()
    task_update = WrappedTaskUpdate(
        item=TaskUpdate(
            id,
            title=title,
            dartboard=dartboard_title,
            status=status_title,
            assignees=assignee_emails if assignee_emails is not None else UNSET,
            tags=tag_titles if tag_titles is not None else UNSET,
            priority=_normalize_priority(priority_int),
            size=size_int,
            due_at=_get_due_at_from_str_arg(due_at_str),
        )
    )
    task = dart.update_task(id, task_update).item

    _log(f"Updated task\n\n  {task.title}\n  {task.html_url}\n  ID: {task.id}\n")
    _log("Done.")
    return task


def delete_task(id: str) -> Task:
    dart = Dart()
    task = dart.delete_task(id).item

    _log(f"Deleted task\n\n  {task.title}\n  {task.html_url}\n  ID: {task.id}\n")
    _log("Done.")
    return task


def create_doc(
    title: str,
    *,
    folder: Union[str, Unset] = UNSET,
    text: Union[str, Unset] = UNSET,
) -> Doc:
    dart = Dart()
    doc_create = WrappedDocCreate(item=DocCreate(title=title, folder=folder, text=text))
    doc = dart.create_doc(doc_create).item

    _log(f"Created doc\n\n  {doc.title}\n  {doc.html_url}\n  ID: {doc.id}\n")
    _log("Done.")
    return doc


def update_doc(
    id: str,
    *,
    title: str,
    folder: Union[str, Unset] = UNSET,
    text: Union[str, Unset] = UNSET,
) -> Doc:
    dart = Dart()
    doc_update = WrappedDocUpdate(item=DocUpdate(id, title=title, folder=folder, text=text))
    doc = dart.update_doc(id, doc_update).item

    _log(f"Updated doc\n\n  {doc.title}\n  {doc.html_url}\n  ID: {doc.id}\n")
    _log("Done.")
    return doc


def delete_doc(id: str) -> Doc:
    dart = Dart()
    doc = dart.delete_doc(id).item

    _log(f"Deleted doc\n\n  {doc.title}\n  {doc.html_url}\n  ID: {doc.id}\n")
    _log("Done.")
    return doc


def create_comment(id: str, text: str) -> Comment:
    dart = Dart()
    comment_create = WrappedCommentCreate(item=CommentCreate(task_id=id, text=text))
    comment = dart.create_comment(comment_create).item
    _log(f"Created comment\n\n  {comment.html_url}\n  ID: {comment.id}\n")
    _log("Done.")
    return comment


def _add_standard_task_arguments(parser: ArgumentParser) -> None:
    parser.add_argument(
        "-d",
        "--dartboard",
        dest="dartboard_title",
        help="dartboard title",
        default=UNSET,
    )
    parser.add_argument(
        "-s",
        "--status",
        dest="status_title",
        help="status title",
        default=UNSET,
    )
    parser.add_argument(
        "-a",
        "--assignee",
        dest="assignee_emails",
        nargs="*",
        action="extend",
        help="assignee email(s)",
    )
    parser.add_argument(
        "-t",
        "--tag",
        dest="tag_titles",
        nargs="*",
        action="extend",
        help="tag title(s)",
    )
    parser.add_argument(
        "-p",
        "--priority",
        dest="priority_int",
        type=int,
        choices=_PRIORITY_MAP.keys(),
        help="priority",
        default=UNSET,
    )
    parser.add_argument(
        "-i",
        "--size",
        dest="size_int",
        type=int,
        choices=_SIZES,
        help="size",
        default=UNSET,
    )
    parser.add_argument(
        "-r",
        "--duedate",
        dest="due_at_str",
        help="due date",
        default=UNSET,
    )


def cli() -> None:
    signal.signal(signal.SIGINT, _exit_gracefully)
    global _is_cli
    _is_cli = True

    print_version_update_message_maybe()

    if _VERSION_CMD in sys.argv[1:]:
        print_version()
        return

    parser = ArgumentParser(prog=_PROG, description="A CLI to interact with Dart")
    metavar = ",".join(
        [
            _LOGIN_CMD,
            _CREATE_TASK_CMD,
            _UPDATE_TASK_CMD,
            _DELETE_TASK_CMD,
            _BEGIN_TASK_CMD,
            _CREATE_DOC_CMD,
            _UPDATE_DOC_CMD,
            _DELETE_DOC_CMD,
            _CREATE_COMMENT_CMD,
        ]
    )
    subparsers = parser.add_subparsers(
        title="command",
        required=True,
        metavar=f"{{{metavar}}}",
    )

    set_host_parser = subparsers.add_parser(_SET_HOST_CMD, aliases=["hs"])
    set_host_parser.add_argument("host", help="the new host: {prod|stag|dev|[URL]}")
    set_host_parser.set_defaults(func=set_host)

    login_parser = subparsers.add_parser(_LOGIN_CMD, aliases=["l"], help="login")
    login_parser.add_argument("-t", "--token", dest="token", help="your authentication token")
    login_parser.set_defaults(func=login)

    create_task_parser = subparsers.add_parser(
        _CREATE_TASK_CMD, aliases=["tc"], help=_HELP_TEXT_TO_COMMAND[_CREATE_TASK_CMD]
    )
    create_task_parser.add_argument("title", help="title of the task")
    create_task_parser.add_argument(
        "-b",
        "--begin",
        dest="should_begin",
        action="store_true",
        help="begin work on the task after creation",
    )
    _add_standard_task_arguments(create_task_parser)
    create_task_parser.set_defaults(func=create_task)

    update_task_parser = subparsers.add_parser(
        _UPDATE_TASK_CMD, aliases=["tu"], help=_HELP_TEXT_TO_COMMAND[_UPDATE_TASK_CMD]
    )
    update_task_parser.add_argument("id", help="ID of the task")
    update_task_parser.add_argument("-e", "--title", dest="title", help="task title", default=UNSET)
    _add_standard_task_arguments(update_task_parser)
    update_task_parser.set_defaults(func=update_task)

    delete_task_parser = subparsers.add_parser(
        _DELETE_TASK_CMD, aliases=["td"], help=_HELP_TEXT_TO_COMMAND[_DELETE_TASK_CMD]
    )
    delete_task_parser.add_argument("id", help="ID of the task")
    delete_task_parser.set_defaults(func=delete_task)

    begin_task_parser = subparsers.add_parser(_BEGIN_TASK_CMD, aliases=["tb"], help="begin work on a task")
    begin_task_parser.set_defaults(func=begin_task)

    create_doc_parser = subparsers.add_parser(
        _CREATE_DOC_CMD, aliases=["dc"], help=_HELP_TEXT_TO_COMMAND[_CREATE_DOC_CMD]
    )
    create_doc_parser.add_argument("title", help="title of the doc")
    create_doc_parser.add_argument("-f", "--folder", dest="folder", help="doc folder", default=UNSET)
    create_doc_parser.add_argument("-t", "--text", dest="text", help="doc text", default=UNSET)
    create_doc_parser.set_defaults(func=create_doc)

    update_doc_parser = subparsers.add_parser(
        _UPDATE_DOC_CMD, aliases=["du"], help=_HELP_TEXT_TO_COMMAND[_UPDATE_DOC_CMD]
    )
    update_doc_parser.add_argument("id", help="ID of the doc")
    update_doc_parser.add_argument("-e", "--title", dest="title", help="doc title", default=UNSET)
    update_doc_parser.add_argument("-f", "--folder", dest="folder", help="doc folder", default=UNSET)
    update_doc_parser.add_argument("-t", "--text", dest="text", help="doc text", default=UNSET)
    update_doc_parser.set_defaults(func=update_doc)

    delete_doc_parser = subparsers.add_parser(
        _DELETE_DOC_CMD, aliases=["dd"], help=_HELP_TEXT_TO_COMMAND[_DELETE_DOC_CMD]
    )
    delete_doc_parser.add_argument("id", help="ID of the doc")
    delete_doc_parser.set_defaults(func=delete_doc)

    create_comment_parser = subparsers.add_parser(
        _CREATE_COMMENT_CMD, aliases=["cc"], help=_HELP_TEXT_TO_COMMAND[_CREATE_COMMENT_CMD]
    )
    create_comment_parser.add_argument("id", help="ID of the task")
    create_comment_parser.add_argument("text", help="text of the comment")
    create_comment_parser.set_defaults(func=create_comment)

    args = vars(parser.parse_args())
    func = args.pop("func")
    func(**args)



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define

T = TypeVar("T", bound="TimeTrackingEntry")


@_attrs_define
class TimeTrackingEntry:
    """
    Attributes:
        user_duid (str):
        started_at (str):
        finished_at (Union[None, str]):
    """

    user_duid: str
    started_at: str
    finished_at: Union[None, str]

    def to_dict(self) -> dict[str, Any]:
        user_duid = self.user_duid

        started_at = self.started_at

        finished_at: Union[None, str]
        finished_at = self.finished_at

        field_dict: dict[str, Any] = {}
        field_dict.update(
            {
                "userDuid": user_duid,
                "startedAt": started_at,
                "finishedAt": finished_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        user_duid = d.pop("userDuid")

        started_at = d.pop("startedAt")

        def _parse_finished_at(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        finished_at = _parse_finished_at(d.pop("finishedAt"))

        time_tracking_entry = cls(
            user_duid=user_duid,
            started_at=started_at,
            finished_at=finished_at,
        )

        return time_tracking_entry



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.paginated_comment_list_meta_type_0_applied_default_filters import (
        PaginatedCommentListMetaType0AppliedDefaultFilters,
    )


T = TypeVar("T", bound="PaginatedCommentListMetaType0")


@_attrs_define
class PaginatedCommentListMetaType0:
    """
    Attributes:
        defaults_applied (Union[Unset, bool]): Whether default filters or ordering were applied to the response.
        applied_default_filters (Union[Unset, PaginatedCommentListMetaType0AppliedDefaultFilters]): The default filters
            that were applied automatically, if any.
        applied_default_sorts (Union[Unset, list[str]]): The default ordering fields that were applied automatically, if
            any.
        instructions (Union[Unset, str]): Guidance on how to disable or override default filters and ordering.
    """

    defaults_applied: Union[Unset, bool] = UNSET
    applied_default_filters: Union[Unset, "PaginatedCommentListMetaType0AppliedDefaultFilters"] = UNSET
    applied_default_sorts: Union[Unset, list[str]] = UNSET
    instructions: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        defaults_applied = self.defaults_applied

        applied_default_filters: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.applied_default_filters, Unset):
            applied_default_filters = self.applied_default_filters.to_dict()

        applied_default_sorts: Union[Unset, list[str]] = UNSET
        if not isinstance(self.applied_default_sorts, Unset):
            applied_default_sorts = self.applied_default_sorts

        instructions = self.instructions

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if defaults_applied is not UNSET:
            field_dict["defaultsApplied"] = defaults_applied
        if applied_default_filters is not UNSET:
            field_dict["appliedDefaultFilters"] = applied_default_filters
        if applied_default_sorts is not UNSET:
            field_dict["appliedDefaultSorts"] = applied_default_sorts
        if instructions is not UNSET:
            field_dict["instructions"] = instructions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.paginated_comment_list_meta_type_0_applied_default_filters import (
            PaginatedCommentListMetaType0AppliedDefaultFilters,
        )

        d = dict(src_dict)
        defaults_applied = d.pop("defaultsApplied", UNSET)

        _applied_default_filters = d.pop("appliedDefaultFilters", UNSET)
        applied_default_filters: Union[Unset, PaginatedCommentListMetaType0AppliedDefaultFilters]
        if isinstance(_applied_default_filters, Unset):
            applied_default_filters = UNSET
        else:
            applied_default_filters = PaginatedCommentListMetaType0AppliedDefaultFilters.from_dict(
                _applied_default_filters
            )

        applied_default_sorts = cast(list[str], d.pop("appliedDefaultSorts", UNSET))

        instructions = d.pop("instructions", UNSET)

        paginated_comment_list_meta_type_0 = cls(
            defaults_applied=defaults_applied,
            applied_default_filters=applied_default_filters,
            applied_default_sorts=applied_default_sorts,
            instructions=instructions,
        )

        paginated_comment_list_meta_type_0.additional_properties = d
        return paginated_comment_list_meta_type_0

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains methods for accessing the API"""

from .attachment import add_task_attachment_from_url
from .comment import add_task_comment, list_comments
from .config import get_config
from .dartboard import get_dartboard
from .doc import create_doc, delete_doc, get_doc, list_docs, update_doc
from .folder import get_folder
from .help_center_article import list_help_center_articles
from .skill import retrieve_skill_by_title
from .task import add_task_time_tracking, create_task, delete_task, get_task, list_tasks, move_task, update_task
from .view import get_view



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.attachment import Attachment


T = TypeVar("T", bound="Skill")


@_attrs_define
class Skill:
    """
    Attributes:
        id (str): The universal, unique ID of the skill.
        title (str): The title of the skill, describing the task type.
        prompt_markdown (str): User-defined instructions for performing this skill in markdown format.
        attachments (list['Attachment']): Attachments linked to the skill, including their accessible URLs.
    """

    id: str
    title: str
    prompt_markdown: str
    attachments: list["Attachment"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        title = self.title

        prompt_markdown = self.prompt_markdown

        attachments = []
        for attachments_item_data in self.attachments:
            attachments_item = attachments_item_data.to_dict()
            attachments.append(attachments_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "title": title,
                "promptMarkdown": prompt_markdown,
                "attachments": attachments,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.attachment import Attachment

        d = dict(src_dict)
        id = d.pop("id")

        title = d.pop("title")

        prompt_markdown = d.pop("promptMarkdown")

        attachments = []
        _attachments = d.pop("attachments")
        for attachments_item_data in _attachments:
            attachments_item = Attachment.from_dict(attachments_item_data)

            attachments.append(attachments_item)

        skill = cls(
            id=id,
            title=title,
            prompt_markdown=prompt_markdown,
            attachments=attachments,
        )

        skill.additional_properties = d
        return skill

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TaskMove")


@_attrs_define
class TaskMove:
    """
    Attributes:
        before_task_id (Union[None, Unset, str]): Move the task immediately before this task. For example: if tasks are
            [A, B, C], then beforeTaskId=B produces [A, moved_task, B, C]. Use null to move to the beginning (first
            position).
        after_task_id (Union[None, Unset, str]): Move the task immediately after this task. For example: if tasks are
            [A, B, C], then afterTaskId=B produces [A, B, moved_task, C]. Use null to move to the end (last position).
    """

    before_task_id: Union[None, Unset, str] = UNSET
    after_task_id: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        before_task_id: Union[None, Unset, str]
        if isinstance(self.before_task_id, Unset):
            before_task_id = UNSET
        else:
            before_task_id = self.before_task_id

        after_task_id: Union[None, Unset, str]
        if isinstance(self.after_task_id, Unset):
            after_task_id = UNSET
        else:
            after_task_id = self.after_task_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if before_task_id is not UNSET:
            field_dict["beforeTaskId"] = before_task_id
        if after_task_id is not UNSET:
            field_dict["afterTaskId"] = after_task_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_before_task_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        before_task_id = _parse_before_task_id(d.pop("beforeTaskId", UNSET))

        def _parse_after_task_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        after_task_id = _parse_after_task_id(d.pop("afterTaskId", UNSET))

        task_move = cls(
            before_task_id=before_task_id,
            after_task_id=after_task_id,
        )

        task_move.additional_properties = d
        return task_move

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import hashlib
import hmac
import os

_WEBHOOK_SECRET = os.environ.get("DART_WEBHOOK_SECRET")
_ENCODED_WEBHOOK_SECRET = _WEBHOOK_SECRET.encode() if _WEBHOOK_SECRET is not None else None


def is_signature_correct(payload: bytes, signature: str) -> bool:
    if _ENCODED_WEBHOOK_SECRET is None:
        raise RuntimeError("DART_WEBHOOK_SECRET environment variable is not set")
    expected_signature = hmac.new(_ENCODED_WEBHOOK_SECRET, payload, hashlib.sha256).hexdigest()
    try:
        return hmac.compare_digest(expected_signature, signature)
    except TypeError:
        return False



import datetime
from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_comments_o_item import ListCommentsOItem
from ...models.paginated_comment_list import PaginatedCommentList
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    author: Union[Unset, str] = UNSET,
    author_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListCommentsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    published_at: Union[Unset, datetime.datetime] = UNSET,
    published_at_after: Union[Unset, datetime.datetime] = UNSET,
    published_at_before: Union[Unset, datetime.datetime] = UNSET,
    task: Union[Unset, str] = UNSET,
    task_id: str,
    text: Union[Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["author"] = author

    params["author_id"] = author_id

    params["ids"] = ids

    params["limit"] = limit

    json_o: Union[Unset, list[str]] = UNSET
    if not isinstance(o, Unset):
        json_o = []
        for o_item_data in o:
            o_item = o_item_data.value
            json_o.append(o_item)

    params["o"] = json_o

    params["offset"] = offset

    params["parent_id"] = parent_id

    json_published_at: Union[Unset, str] = UNSET
    if not isinstance(published_at, Unset):
        json_published_at = published_at.isoformat()
    params["published_at"] = json_published_at

    json_published_at_after: Union[Unset, str] = UNSET
    if not isinstance(published_at_after, Unset):
        json_published_at_after = published_at_after.isoformat()
    params["published_at_after"] = json_published_at_after

    json_published_at_before: Union[Unset, str] = UNSET
    if not isinstance(published_at_before, Unset):
        json_published_at_before = published_at_before.isoformat()
    params["published_at_before"] = json_published_at_before

    params["task"] = task

    params["task_id"] = task_id

    params["text"] = text

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/comments/list",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[PaginatedCommentList]:
    if response.status_code == 200:
        response_200 = PaginatedCommentList.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[PaginatedCommentList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    author: Union[Unset, str] = UNSET,
    author_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListCommentsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    published_at: Union[Unset, datetime.datetime] = UNSET,
    published_at_after: Union[Unset, datetime.datetime] = UNSET,
    published_at_before: Union[Unset, datetime.datetime] = UNSET,
    task: Union[Unset, str] = UNSET,
    task_id: str,
    text: Union[Unset, str] = UNSET,
) -> Response[PaginatedCommentList]:
    """List comments for a task with filtering options. Filter by author, text content, or date range. Sort
    by date or hierarchical thread order. Task ID required. Supports pagination.

    Args:
        author (Union[Unset, str]):
        author_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListCommentsOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        published_at (Union[Unset, datetime.datetime]):
        published_at_after (Union[Unset, datetime.datetime]):
        published_at_before (Union[Unset, datetime.datetime]):
        task (Union[Unset, str]):
        task_id (str):
        text (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedCommentList]
    """

    kwargs = _get_kwargs(
        author=author,
        author_id=author_id,
        ids=ids,
        limit=limit,
        o=o,
        offset=offset,
        parent_id=parent_id,
        published_at=published_at,
        published_at_after=published_at_after,
        published_at_before=published_at_before,
        task=task,
        task_id=task_id,
        text=text,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    author: Union[Unset, str] = UNSET,
    author_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListCommentsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    published_at: Union[Unset, datetime.datetime] = UNSET,
    published_at_after: Union[Unset, datetime.datetime] = UNSET,
    published_at_before: Union[Unset, datetime.datetime] = UNSET,
    task: Union[Unset, str] = UNSET,
    task_id: str,
    text: Union[Unset, str] = UNSET,
) -> Optional[PaginatedCommentList]:
    """List comments for a task with filtering options. Filter by author, text content, or date range. Sort
    by date or hierarchical thread order. Task ID required. Supports pagination.

    Args:
        author (Union[Unset, str]):
        author_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListCommentsOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        published_at (Union[Unset, datetime.datetime]):
        published_at_after (Union[Unset, datetime.datetime]):
        published_at_before (Union[Unset, datetime.datetime]):
        task (Union[Unset, str]):
        task_id (str):
        text (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedCommentList
    """

    return sync_detailed(
        client=client,
        author=author,
        author_id=author_id,
        ids=ids,
        limit=limit,
        o=o,
        offset=offset,
        parent_id=parent_id,
        published_at=published_at,
        published_at_after=published_at_after,
        published_at_before=published_at_before,
        task=task,
        task_id=task_id,
        text=text,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    author: Union[Unset, str] = UNSET,
    author_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListCommentsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    published_at: Union[Unset, datetime.datetime] = UNSET,
    published_at_after: Union[Unset, datetime.datetime] = UNSET,
    published_at_before: Union[Unset, datetime.datetime] = UNSET,
    task: Union[Unset, str] = UNSET,
    task_id: str,
    text: Union[Unset, str] = UNSET,
) -> Response[PaginatedCommentList]:
    """List comments for a task with filtering options. Filter by author, text content, or date range. Sort
    by date or hierarchical thread order. Task ID required. Supports pagination.

    Args:
        author (Union[Unset, str]):
        author_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListCommentsOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        published_at (Union[Unset, datetime.datetime]):
        published_at_after (Union[Unset, datetime.datetime]):
        published_at_before (Union[Unset, datetime.datetime]):
        task (Union[Unset, str]):
        task_id (str):
        text (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PaginatedCommentList]
    """

    kwargs = _get_kwargs(
        author=author,
        author_id=author_id,
        ids=ids,
        limit=limit,
        o=o,
        offset=offset,
        parent_id=parent_id,
        published_at=published_at,
        published_at_after=published_at_after,
        published_at_before=published_at_before,
        task=task,
        task_id=task_id,
        text=text,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    author: Union[Unset, str] = UNSET,
    author_id: Union[Unset, str] = UNSET,
    ids: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    o: Union[Unset, list[ListCommentsOItem]] = UNSET,
    offset: Union[Unset, int] = UNSET,
    parent_id: Union[Unset, str] = UNSET,
    published_at: Union[Unset, datetime.datetime] = UNSET,
    published_at_after: Union[Unset, datetime.datetime] = UNSET,
    published_at_before: Union[Unset, datetime.datetime] = UNSET,
    task: Union[Unset, str] = UNSET,
    task_id: str,
    text: Union[Unset, str] = UNSET,
) -> Optional[PaginatedCommentList]:
    """List comments for a task with filtering options. Filter by author, text content, or date range. Sort
    by date or hierarchical thread order. Task ID required. Supports pagination.

    Args:
        author (Union[Unset, str]):
        author_id (Union[Unset, str]):
        ids (Union[Unset, str]):
        limit (Union[Unset, int]):
        o (Union[Unset, list[ListCommentsOItem]]):
        offset (Union[Unset, int]):
        parent_id (Union[Unset, str]):
        published_at (Union[Unset, datetime.datetime]):
        published_at_after (Union[Unset, datetime.datetime]):
        published_at_before (Union[Unset, datetime.datetime]):
        task (Union[Unset, str]):
        task_id (str):
        text (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PaginatedCommentList
    """

    return (
        await asyncio_detailed(
            client=client,
            author=author,
            author_id=author_id,
            ids=ids,
            limit=limit,
            o=o,
            offset=offset,
            parent_id=parent_id,
            published_at=published_at,
            published_at_after=published_at_after,
            published_at_before=published_at_before,
            task=task,
            task_id=task_id,
            text=text,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TaskCustomPropertiesType0")


@_attrs_define
class TaskCustomPropertiesType0:
    """Custom properties as a dict mapping property NAME to value. Use exact property names from workspace config (e.g.,
    {"customCheckboxProperty": true, "customTextProperty": "Some text"}). Property names are case-sensitive.

        Example:
            {'customCheckboxProperty': True, 'customDatesProperty': '2025-05-10', 'customDatesPropertyWithRange':
                ['2025-05-01', '2025-05-30'], 'customMultiselectProperty': ['frontend', 'bug'],
                'customNumberPropertyWithIntegerFormat': 5, 'customNumberPropertyWithPercentageFormat': 75,
                'customNumberPropertyWithDollarsFormat': 1500.5, 'customSelectProperty': 'In Progress', 'customStatusProperty':
                'Blocked', 'customTextProperty': 'This task requires additional review from the design team',
                'customUserProperty': 'john.doe@example.com', 'customMultipleUserProperty': ['john.doe@example.com', 'Alice
                Smith']}

    """

    additional_properties: dict[str, Union[None, bool, float, int, list[Union[None, str]], list[str], str]] = (
        _attrs_field(init=False, factory=dict)
    )

    def to_dict(self) -> dict[str, Any]:
        field_dict: dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            if isinstance(prop, list):
                field_dict[prop_name] = []
                for additional_property_type_1_type_0_item_data in prop:
                    additional_property_type_1_type_0_item: Union[None, str]
                    additional_property_type_1_type_0_item = additional_property_type_1_type_0_item_data
                    field_dict[prop_name].append(additional_property_type_1_type_0_item)

            elif isinstance(prop, list):
                field_dict[prop_name] = prop

            elif isinstance(prop, list):
                field_dict[prop_name] = prop

            else:
                field_dict[prop_name] = prop

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        task_custom_properties_type_0 = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():

            def _parse_additional_property(
                data: object,
            ) -> Union[None, bool, float, int, list[Union[None, str]], list[str], str]:
                if data is None:
                    return data
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    additional_property_type_1_type_0 = []
                    _additional_property_type_1_type_0 = data
                    for additional_property_type_1_type_0_item_data in _additional_property_type_1_type_0:

                        def _parse_additional_property_type_1_type_0_item(
                            data: object,
                        ) -> Union[None, str]:
                            if data is None:
                                return data
                            return cast(Union[None, str], data)

                        additional_property_type_1_type_0_item = _parse_additional_property_type_1_type_0_item(
                            additional_property_type_1_type_0_item_data
                        )

                        additional_property_type_1_type_0.append(additional_property_type_1_type_0_item)

                    return additional_property_type_1_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    additional_property_type_3 = cast(list[str], data)

                    return additional_property_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    additional_property_type_11 = cast(list[str], data)

                    return additional_property_type_11
                except:  # noqa: E722
                    pass
                return cast(
                    Union[None, bool, float, int, list[Union[None, str]], list[str], str],
                    data,
                )

            additional_property = _parse_additional_property(prop_dict)

            additional_properties[prop_name] = additional_property

        task_custom_properties_type_0.additional_properties = additional_properties
        return task_custom_properties_type_0

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Union[None, bool, float, int, list[Union[None, str]], list[str], str]:
        return self.additional_properties[key]

    def __setitem__(
        self,
        key: str,
        value: Union[None, bool, float, int, list[Union[None, str]], list[str], str],
    ) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListDocsOItem(str, Enum):
    CREATED_AT = "created_at"
    FOLDER_ORDER = "folder__order"
    ORDER = "order"
    TITLE = "title"
    UPDATED_AT = "updated_at"
    CREATED_AT_DESC = "-created_at"
    FOLDER_ORDER_DESC = "-folder__order"
    ORDER_DESC = "-order"
    TITLE_DESC = "-title"
    UPDATED_AT_DESC = "-updated_at"

    def __str__(self) -> str:
        return str(self.value)
